package xyz.etztech.minecraftmanager.objects;

import org.apache.commons.lang.StringUtils;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import xyz.etztech.core.web.CoreWeb;
import xyz.etztech.minecraftmanager.MinecraftManager;

import java.util.*;

public class OreAlert {
    private Map<UUID, ArrayList<Date>> players = new HashMap<>();
    private List<String> diamonds = new ArrayList<>();
    private List<Strike> queue = new ArrayList<>();

    private MinecraftManager plugin;

    public OreAlert(MinecraftManager plugin) {
        this.plugin = plugin;
    }

    public List<Strike> getQueue() {
        return this.queue;
    }

    public void addStrike(UUID uuid) {
        ArrayList<Date> strikes = players.getOrDefault(uuid, new ArrayList<>());
        strikes.add(new Date());
        players.put(uuid, strikes);
    }

    public int getStrikes(UUID uuid) {
        return players.getOrDefault(uuid, new ArrayList<>()).size();
    }

    public void alert(String username, int strikes, boolean ping) {
        // <username> has found <strikes> diamond ore veins.
        String message = username + " has found " + strikes + " diamond ore veins.";

        for (Player player : Bukkit.getOnlinePlayers()) {
            if (player.hasPermission("minecraftmanager.orealert")) {
                player.sendMessage(ChatColor.AQUA + message);
            }
        }

        // Webhook
        String webhook = plugin.getConfig().getString("orealert.notify.webhook", "");
        if (StringUtils.isNotEmpty(webhook)) {
            String content = message;
            if (ping) {
                content = "@here " + message;
            }
            Map<String, String> data = new HashMap<>();
            data.put("username", "OreAlert");
            data.put("content", content);
            CoreWeb.asyncPost(plugin, webhook, data);
        }
    }

    public void purge(UUID uuid, int purge) {
        Date cutoff = new Date();
        // Cutoff to minutes, subtract purge, back to milliseconds
        cutoff.setTime(((cutoff.getTime()/1000/60) - purge)*60*1000);
        ArrayList<Date> purged = players.getOrDefault(uuid, new ArrayList<>());
        purged.removeIf(date -> date.before(cutoff));
        players.put(uuid, purged);
    }

    public boolean addDiamond(String location) {
        if (diamonds.contains(location)) {
            return false;
        }
        diamonds.add(location);
        return true;
    }


    public void monitor() {
        Strike strike;
        if (queue.size() > 0) {
            strike = queue.get(0);
            if (addDiamond(getLocationString(strike.getBlock()))) {
                addStrike(strike.getPlayer().getUniqueId());
                purge(strike.getPlayer().getUniqueId(), plugin.getConfig().getInt("orealert.purge", 30));

                int alertable = getStrikes(strike.getPlayer().getUniqueId());

                double start = (double) alertable / plugin.getConfig().getInt("orealert.notify.start", 5);
                // Start
                if (start == 1) {
                    alert(strike.getPlayer().getName(), alertable, true);
                } else if (start > 1) {
                    // Ping
                    if (alertable % plugin.getConfig().getInt("orealert.notify.ping", 5) == 0) {
                        alert(strike.getPlayer().getName(), alertable, true);
                        // Alert
                    } else if (alertable % plugin.getConfig().getInt("orealert.notify.each", 1) == 0) {
                        alert(strike.getPlayer().getName(), alertable, false);
                    }
                }

                for (Block radiusBlock : getBlocks(strike.getBlock(), 5)) {
                    if (Material.DIAMOND_ORE == radiusBlock.getType()) {
                        addDiamond(getLocationString(radiusBlock));
                    }
                }

            }
            queue.remove(0);
        }
    }

    private List<Block> getBlocks(Block start, int radius) {
        if (radius < 0) {
            return new ArrayList<>(0);
        }
        int iterations = (radius * 2) + 1;
        List<Block> blocks = new ArrayList<Block>(iterations * iterations * iterations);
        for (int x = -radius; x <= radius; x++) {
            for (int y = -radius; y <= radius; y++) {
                for (int z = -radius; z <= radius; z++) {
                    blocks.add(start.getRelative(x, y, z));
                }
            }
        }
        return blocks;
    }

    private String getLocationString(Block block) {
        int X = block.getX();
        int Y = block.getY();
        int Z = block.getZ();

        String x = String.valueOf(X);
        String y = String.valueOf(Y);
        String z = String.valueOf(Z);

        return x + y + z;
    }

    public static class Strike {
        private Player player;
        private Block block;

        public Strike(Player player, Block block) {
            this.player = player;
            this.block = block;
        }

        public Player getPlayer() {
            return player;
        }

        public Block getBlock() {
            return block;
        }
    }

}
